/*
	NAME:	Simple Direct3D9 Application
	DESC:	Experimentation with D3D9

	AUTHOR:	Jack Hoxley
	DATE:	January 2003
*/

//a necessary include when writing Win32 programs
#include <Windows.h>
#include <exception>
#include <stdio.h>
#include "Gfx3D.h"
#include "Gfx3DException.h"

#define WINDOWED_MODE	//comment this out to get fullscreen.

//global variables. ugly as hell, but I cant
//be bothered to sort them out right now!
Gfx3D *pGFXEngine;

LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
		//we aren't interested in many messages for this sample app.
        case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;
			break;

		case WM_KEYDOWN:
			//a key was depressed
			//wParam:	virtual key-code pressed (VK_***)
			switch ( wParam ) {
				case VK_ESCAPE:
					//exit program
					PostQuitMessage( 0 );
					break;
				case VK_F1:
					//wireframe
					pGFXEngine->Wireframe(!pGFXEngine->Wireframe());
					break;
				case VK_F2:
					//textures
					pGFXEngine->UseTextures(!pGFXEngine->UseTextures());
					break;
				case VK_F3:
					//culling
					pGFXEngine->BackFaceCulling(!pGFXEngine->BackFaceCulling());
					break;
				case VK_F4:
					//frame rate
					pGFXEngine->DrawFrameRate(!pGFXEngine->DrawFrameRate());
					break;
				default:
					break;
			}
			return 0;
			break;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}


//WinMain - the entry point for this application. execution starts here
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT ) {
	//0. Any variables;
	WNDCLASSEX window;								//holds everything we know about our window TEMPLATE.
	HWND hWnd;										//holds a reference to the CREATED instance of the window
	
	//1. Configure the window class
	ZeroMemory( &window, sizeof(WNDCLASSEX) );		//set all unneeded members to NULL/0
	window.cbSize = sizeof(WNDCLASSEX);				//store the size of the variable
	window.lpszClassName = "Direct3D9Cpp";			//give it a unique name. the user wont see this.
	window.hInstance = GetModuleHandle(NULL);		//get a default module handle.
	window.lpfnWndProc = MsgProc;					//IMPORTANT: tell Win32 where to send messages.

	//2. register it with the system
	RegisterClassEx( &window );						//register the class with the system.

	//3. create the window for use
	hWnd = CreateWindow("Direct3D9Cpp",				//refer back to the name we gave the class earlier
						"Simple D3D9 Sample",		//this is the window caption - the user sees this.
						WS_OVERLAPPEDWINDOW,		//style of window to use.
						0, 0,						// TOP LEFT COORDINATE
						400,300,					// BOTTOM RIGHT COORDINATE
						GetDesktopWindow(),			//parent window will be the windows desktop
						NULL,						//we aren't using menu's so we pass NULL
						window.hInstance,			//handle to the original class
						NULL);						//not important. for MDI windows.

	//4. show the window on screen
	ShowWindow(hWnd,SW_SHOWDEFAULT);				//display the window on screen.
	UpdateWindow( hWnd );							//update it / send it initial msgs.

	//5. create the graphics engine
	try {
#ifdef WINDOWED_MODE
		pGFXEngine = new Gfx3D(hWnd);
#else
		if(Gfx3D::IsDisplayModeOkay(1024,768,32))
			pGFXEngine = new Gfx3D(hWnd, 1024,768,32);
		else
			if(Gfx3D::IsDisplayModeOkay(1024,768,16))
				pGFXEngine = new Gfx3D(hWnd, 1024,768,16);
			else
				if(Gfx3D::IsDisplayModeOkay(640,480,32))
					pGFXEngine = new Gfx3D(hWnd, 640,480,32);
				else
					if(Gfx3D::IsDisplayModeOkay(640,480,16))
						pGFXEngine = new Gfx3D(hWnd, 640,480,16);
					else
						//fail!!!
						throw Gfx3DException("Could not find a compatable fullscreen display mode","Core.cpp",__LINE__);
#endif

	} catch ( Gfx3DException x ) {
		//catch a specific engine-specified error message
		//create a new ptr to hold the full message
		char *pMsg = new char[200];
		//compose the message
		sprintf(pMsg,"Gfx3DException was caught: \n%s\n\nThis application will now exit",x.GetErrMsg());	
		//output it to the screen for the user
		MessageBox( hWnd, pMsg, "Error", MB_OK );					
		delete pMsg;
		//clean up and exit:
		delete pGFXEngine;
		UnregisterClass( "Direct3D9Cpp", window.hInstance );
		return 0;
	} catch ( exception err ) {
		//catch a generic erorr message
		MessageBox(hWnd,err.what(),"Error",MB_OK);
		MessageBox(hWnd,"Unknown exception was caught; this application will now exit.","Error",MB_OK);
		//clean up and exit:
		delete pGFXEngine;
		UnregisterClass( "Direct3D9Cpp", window.hInstance );
		return 0;
	}

	pGFXEngine->BackFaceCulling(true);
	pGFXEngine->Wireframe(false);
	pGFXEngine->UseTextures(true);
	pGFXEngine->DrawFrameRate(true);

	//6. enter the main application loop
	MSG winMsg;														//create a msg structure that we can use
	ZeroMemory( &winMsg, sizeof(winMsg) );							//clear its memory
	while (winMsg.message != WM_QUIT) {								//loop until we're told to quit
		if (PeekMessage( &winMsg, NULL, 0U, 0U, PM_REMOVE )) {		//check for any new messages
			TranslateMessage( &winMsg );							//process them.
			DispatchMessage( &winMsg );
		} else {													//no windows messages. lets get on with D3D graphics
			pGFXEngine->OneFrameUpdate();							//update maths/physics etc... for the current frame.
			pGFXEngine->OneFrameRender();							//render the current frame to the screen.
		}
	}

	//7. tidy up
	delete pGFXEngine;
	UnregisterClass( "Direct3D9Cpp", window.hInstance );			//tell the system we're no longer using the window
	return 0;														//exit the program.
}