using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace DX9Test
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private GraphicsEngine engine;
		private bool running = false;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				engine.Dispose();

				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// Form1
			// 
			this.AutoScale = false;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.Color.Black;
			this.ClientSize = new System.Drawing.Size(566, 436);
			this.ControlBox = false;
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "Form1";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Simple Direct3D9";
			this.Load += new System.EventHandler(this.Form1_Load);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void Form1_Load(object sender, System.EventArgs e)
		{
			this.Show();
			Application.DoEvents();

			DialogResult result = MessageBox.Show(this, "Use full-screen mode?", "Full-Screen Option", 
				MessageBoxButtons.YesNo, MessageBoxIcon.Question);

			if (result == DialogResult.No)
			{
				// create the windowed engine
				engine = new GraphicsEngine(this);
			}
			else
			{
				Cursor.Hide();

				// Run through 6 full-screen modes and choose the best.
				// It is more common to present a list of avialable modes to
				// the user, but we don't want to because we're lazy.
				if (!SetFullScreenDisplayMode(1280, 1024, 32) &&
					!SetFullScreenDisplayMode(1280, 1024, 16) &&
					!SetFullScreenDisplayMode(1024, 768, 32) &&
					!SetFullScreenDisplayMode(1024, 768, 16) &&
					!SetFullScreenDisplayMode(640, 480, 32) &&
					!SetFullScreenDisplayMode(640, 480, 16))
				{
					throw new ApplicationException("No supported display modes found.");
				}
			}

			// set the initial properties for the engine
			engine.BackFaceCulling = false;
			engine.DrawFrameRate = true;
			engine.UseTextures = true;
			engine.WireFrame = false;

			// as soon as this turns to false we bail out and exit the program
			running = true;

			// the main loop
			while (running)
			{
				// these two functions do all the work
				engine.OneFrameUpdate();
				engine.OneFrameRender();
				Application.DoEvents();
			}

			// if we've left the loop then the application has finished
			engine.Dispose();
			this.Close();
		}

		private bool SetFullScreenDisplayMode(int width, int height, int depth)
		{
			if (GraphicsEngine.IsDisplayModeOkay(width, height, depth))
			{
				engine = new GraphicsEngine(this, width, height, depth);
				return true;
			}

			return false;
		}

		protected override void OnKeyDown(KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.Escape:
					running = false;
					break;
				case Keys.F1:
					engine.WireFrame = !engine.WireFrame;
					break;
				case Keys.F2:
					engine.UseTextures = !engine.UseTextures;
					break;
				case Keys.F3:
					engine.BackFaceCulling = !engine.BackFaceCulling;
					break;
				case Keys.F4:
					engine.DrawFrameRate = !engine.DrawFrameRate;
					break;
			}

			base.OnKeyDown(e);
		}

	}
}
