/*
	
	TITLE:		Gfx3D.h
	DESC:		Interface definition header for simple Direct3D9 sample program
	DATE:		January 2003

	AUTHOR:		Jack Hoxley

*/


#ifndef INC_GFX3D_H	//check for inclusion guard
#define INC_GFX3D_H	//declare inclusion guard

//include necessary system headers:
#include <math.h>
#include <d3dx9.h>

class Gfx3D {	
private:
	//--------------//
	//	VARIABLES	//
	//--------------//

	//core objects
	LPDIRECT3D9 pD3D9;
	LPDIRECT3DDEVICE9 pDev;

	//matrices
	D3DXMATRIXA16 MatWorld;
	D3DXMATRIXA16 MatView;
	D3DXMATRIXA16 MatProj;
	D3DXMATRIXA16 MatCube1;
	D3DXMATRIXA16 MatCube2;

	//geometry
	LPDIRECT3DVERTEXBUFFER9 pVBCube;
	LPDIRECT3DVERTEXBUFFER9 pVBMenu;

	//vertex structs
	struct vertex3D {
		D3DXVECTOR3 p;	//position
		D3DXVECTOR2 t;	//texture
	};
	static const DWORD FVF_VERTEX3D =  D3DFVF_XYZ|D3DFVF_TEX1;

	struct vertex2D {
		D3DXVECTOR4 p;	//position
		D3DXVECTOR2 t;	//texture
	};
	static const DWORD FVF_VERTEX2D = D3DFVF_XYZRHW|D3DFVF_TEX1;

	//textures
	LPDIRECT3DTEXTURE9 pTexCube1;
	LPDIRECT3DTEXTURE9 pTexCube2;
	LPDIRECT3DTEXTURE9 pTexMenu;

	//fonts
	HFONT hMainFont;
	LPD3DXFONT lpFont;

	//per-frame variables
	DWORD dwLastFrameUpdate;
	float Cube1Angle;
	float Cube2Angle;

	//constants
	static const int Cube1Speed = 50;
	static const int Cube2Speed = 30;
	static const int Cube1Size = 1;
	static const int Cube2Size = 2;
	static const int iFPSPollRate = 200;
	static const int iFontSize = 14;

	//control variables
	bool bWireFrame;
	bool bUseTextures;
	bool bCulling;
	bool bDrawFPS;

	//misc variables
	bool bInitOkay;
	DWORD dwLastFPSCheck;
	float fCurrFPS;
    int iCurrCnt;
	char *sDevInfo;
	char *sDispInfo;

	//--------------//
	//	FUNCTIONS	//
	//--------------//
	bool InitialiseDevice(HWND hWnd, int iW, int iH, int iD, bool bWin);
	bool LoadTextures(D3DFORMAT fmtDisplay);
	bool LoadGeometry();

public:
	//----------------------------------//
	//	CONSTRUCTORS AND DESTRUCTORS	//
	//----------------------------------//
	Gfx3D(HWND hWnd);							//windowed mode constructor
	Gfx3D(HWND hWnd, int iW, int iH, int iD);	//fullscreen mode constructor
	~Gfx3D();									//destructor

	//----------------------//
	//	PUBLIC INTERFACES	//
	//----------------------//
	static bool IsDisplayModeOkay(int iW, int iH, int iD);	//checks the parameters and determines if the
													//device supports it as a fullscreen resolution.
	bool OneFrameUpdate();							//updates maths/physics for current frame
	bool OneFrameRender();							//renders the current frame

	//--------------//
	//	PROPERTIES	//
	//--------------//
	void Wireframe(bool bWire);					//set
	bool Wireframe();							//get

	void UseTextures(bool bTex);				//set
	bool UseTextures();							//get

	void BackFaceCulling(bool bCull);			//set
	bool BackFaceCulling();						//get

	void DrawFrameRate(bool bFPS);				//set
	bool DrawFrameRate();						//get
};

#endif	//#ifndef INC_GFX3D_H